@@ -176,7 +176,9 @@ class PyAVReader(BasePyAVReader):
             if len(self.streams) < 2:
                 self._alpha_merger = NotAlphaMerger()
             elif len(self.streams) == 2:
-                self._alpha_merger = AlphaMerger()
+                self._alpha_merger = AlphaMerger(
+                    self.streams[0].format.name, self.streams[1].format.name
+                )
             else:
                 raise NotImplementedError
         return self._alpha_merger
@@ -368,7 +370,7 @@ class PyAVWriter(PyAVInterface):
     @property
     def alpha_extractor(self):
         if self._alpha_extractor is None and self._alpha_stream:
-            self._alpha_extractor = AlphaExtractor(self._width, self._height)
+            self._alpha_extractor = AlphaExtractor()
         return self._alpha_extractor
 
     def array_to_frame(self, array):
@@ -480,6 +482,7 @@ class Filter(ABC):
             self.graph = av.filter.Graph()
             self._rebuild_graph(**kwargs)
             self.graph.configure()
+            self._kwargs = kwargs
 
     @abstractmethod
     def _rebuild_graph(**kwargs):
@@ -542,10 +545,6 @@ class _AlphaExtractor(Filter):
     def __init__(self):
         super().__init__()
 
-        self._width = None
-        self._height = None
-        self._pix_fmt = None
-
     def _rebuild_graph(self, width: int, height: int, pix_fmt: str):
         src = self.graph.add_buffer(
             width=width,
@@ -597,26 +596,26 @@ class BaseAlphaMerger:
 
 
 class AlphaMerger(BaseAlphaMerger):
-    def __init__(self):
-        self.alpha_mergers = defaultdict(_AlphaMerger)
+    def __init__(self, image_pix_fmt: str, alpha_pix_fmt: str):
+        self.alpha_mergers = defaultdict(
+            lambda: _AlphaMerger(image_pix_fmt, alpha_pix_fmt)
+        )
 
     def push_image(self, frame: av.VideoFrame):
-        self.alpha_mergers[(frame.width, frame.height, frame.format.name)].push_image(
-            frame
-        )
+        self.alpha_mergers[(frame.width, frame.height)].push_image(frame)
 
     def push_alpha(self, frame: av.VideoFrame):
-        self.alpha_mergers[(frame.width, frame.height, frame.format.name)].push_alpha(
-            frame
-        )
+        self.alpha_mergers[(frame.width, frame.height)].push_alpha(frame)
 
     def pull(self) -> av.VideoFrame:
-        for key in list(self.alpha_mergers):
+        keys = list(self.alpha_mergers)
+        for key in keys:
             ret = self.alpha_mergers[key].pull()
             if ret is not None:
                 return ret
 
-            del self.alpha_mergers[key]
+            if len(keys) > 1:
+                del self.alpha_mergers[key]
 
     def close(self):
         for it in self.alpha_mergers.values():
@@ -624,22 +623,26 @@ class AlphaMerger(BaseAlphaMerger):
 
 
 class _AlphaMerger(Filter, BaseAlphaMerger):
+    def __init__(self, image_pix_fmt: str, alpha_pix_fmt: str):
+        super().__init__()
+        self.image_pix_fmt = image_pix_fmt
+        self.alpha_pix_fmt = alpha_pix_fmt
+
     def _rebuild_graph(
         self,
         width: int,
         height: int,
-        pix_fmt: str,
     ):
         self.image = self.graph.add_buffer(
             width=width,
             height=height,
-            format=pix_fmt,
+            format=self.image_pix_fmt,
             time_base=fractions.Fraction(1, 1000),
         )
         self.alpha = self.graph.add_buffer(
             width=width,
             height=height,
-            format=pix_fmt,
+            format=self.alpha_pix_fmt,
             time_base=fractions.Fraction(1, 1000),
         )
 
@@ -654,15 +657,11 @@ class _AlphaMerger(Filter, BaseAlphaMerger):
         alphamerge.link_to(self.result)
 
     def push_image(self, frame: av.VideoFrame):
-        self.rebuild_graph(
-            width=frame.width, height=frame.height, pix_fmt=frame.format.name
-        )
+        self.rebuild_graph(width=frame.width, height=frame.height)
         self.image.push(frame)
 
     def push_alpha(self, frame: av.VideoFrame):
-        self.rebuild_graph(
-            width=frame.width, height=frame.height, pix_fmt=frame.format.name
-        )
+        self.rebuild_graph(width=frame.width, height=frame.height)
         self.alpha.push(frame)
 
     def pull(self) -> av.VideoFrame:
@@ -835,7 +834,7 @@ class Overlayer(BaseOverlayer):
         left=0.0,
         top=0.0,
         height=1.0,
-        pix_fmt="yuva420p",
+        pix_fmt: str = "yuva420p",
         mode: str = "straight",
     ):
         dst_size, background_image = get_dst_size(dst_size, background_image)
